@@ -9,15 +9,16 @@
 
 int bwa_bwtsw2(int argc, char *argv[])
 {
+	extern char *bwa_infer_prefix(const char *hint);
 	bsw2opt_t *opt;
 	bwt_t *target;
-	char buf[1024];
+	char buf[1024], *prefix;
 	bntseq_t *bns;
 	int c;
 
 	opt = bsw2_init_opt();
 	srand48(11);
-	while ((c = getopt(argc, argv, "q:r:a:b:t:T:w:d:z:m:s:c:N:Hf:M")) >= 0) {
+	while ((c = getopt(argc, argv, "q:r:a:b:t:T:w:d:z:m:s:c:N:Hf:MI:S")) >= 0) {
 		switch (c) {
 		case 'q': opt->q = atoi(optarg); break;
 		case 'r': opt->r = atoi(optarg); break;
@@ -34,6 +35,8 @@ int bwa_bwtsw2(int argc, char *argv[])
 		case 'M': opt->multi_2nd = 1; break;
 		case 'H': opt->hard_clip = 1; break;
 		case 'f': xreopen(optarg, "w", stdout); break;
+		case 'I': opt->max_ins = atoi(optarg); break;
+		case 'S': opt->skip_sw = 1; break;
 		}
 	}
 	opt->qr = opt->q + opt->r;
@@ -49,9 +52,11 @@ int bwa_bwtsw2(int argc, char *argv[])
 		fprintf(stderr, "         -m FLOAT mask level [%.2f]\n", opt->mask_level);
 		fprintf(stderr, "\n");
 		fprintf(stderr, "         -t INT   number of threads [%d]\n", opt->n_threads);
-        fprintf(stderr, "         -f FILE  file to output results to instead of stdout\n");
+		fprintf(stderr, "         -f FILE  file to output results to instead of stdout\n");
 		fprintf(stderr, "         -H       in SAM output, use hard clipping instead of soft clipping\n");
 		fprintf(stderr, "         -M       mark multi-part alignments as secondary\n");
+		fprintf(stderr, "         -S       skip Smith-Waterman read pairing\n");
+		fprintf(stderr, "         -I INT   ignore pairs with insert >=INT for inferring the size distr [%d]\n", opt->max_ins);
 		fprintf(stderr, "\n");
 		fprintf(stderr, "         -T INT   score threshold divided by a [%d]\n", opt->t);
 		fprintf(stderr, "         -c FLOAT coefficient of length-threshold adjustment [%.1f]\n", opt->coef);
@@ -72,15 +77,19 @@ int bwa_bwtsw2(int argc, char *argv[])
 	opt->t *= opt->a;
 	opt->coef *= opt->a;
 
-	strcpy(buf, argv[optind]); target = bwt_restore_bwt(strcat(buf, ".bwt"));
-	strcpy(buf, argv[optind]); bwt_restore_sa(strcat(buf, ".sa"), target);
-	bns = bns_restore(argv[optind]);
+	if ((prefix = bwa_infer_prefix(argv[optind])) == 0) {
+		fprintf(stderr, "[%s] fail to locate the index\n", __func__);
+		return 0;
+	}
+	strcpy(buf, prefix); target = bwt_restore_bwt(strcat(buf, ".bwt"));
+	strcpy(buf, prefix); bwt_restore_sa(strcat(buf, ".sa"), target);
+	bns = bns_restore(prefix);
 
 	bsw2_aln(opt, bns, target, argv[optind+1], optind+2 < argc? argv[optind+2] : 0);
 
 	bns_destroy(bns);
 	bwt_destroy(target);
-	free(opt);
+	free(opt); free(prefix);
 	
 	return 0;
 }
