@@ -6,7 +6,6 @@ extern "C" {
 #endif
 
 	int bwa_fa2pac(int argc, char *argv[]);
-	int bwa_pac_rev(int argc, char *argv[]);
 	int bwa_pac2cspac(int argc, char *argv[]);
 	int bwa_pac2bwt(int argc, char *argv[]);
 	int bwa_bwtupdate(int argc, char *argv[]);
@@ -22,6 +21,8 @@ extern "C" {
 
 	int bwa_bwtsw2(int argc, char *argv[]);
 
+	int main_fastmap(int argc, char *argv[]);
+
 #ifdef __cplusplus
 }
 #endif
